package com.atguigu.tingshu.payment.service.impl;

import com.atguigu.tingshu.common.constant.SystemConstant;
import com.atguigu.tingshu.common.execption.GuiguException;
import com.atguigu.tingshu.model.payment.PaymentInfo;
import com.atguigu.tingshu.payment.config.WxPayV3Config;
import com.atguigu.tingshu.payment.service.PaymentInfoService;
import com.atguigu.tingshu.payment.service.WxPayService;
import com.wechat.pay.java.core.RSAAutoCertificateConfig;
import com.wechat.pay.java.service.payments.jsapi.JsapiServiceExtension;
import com.wechat.pay.java.service.payments.jsapi.model.*;
import com.wechat.pay.java.service.payments.model.Transaction;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.HashMap;
import java.util.Map;

@Service
@Slf4j
public class WxPayServiceImpl implements WxPayService {

    @Autowired
    private PaymentInfoService paymentInfoService;

    @Autowired
    private RSAAutoCertificateConfig config;

    @Autowired
    private WxPayV3Config wxPayV3Config;

    /**
     * 获取小程序拉起微信支付所需参数
     *
     * @param paymentType 支付类型 1301-订单 1302-充值
     * @param orderNo     交易订单号或充值订单号
     * @return {"timeStamp":"","package":"","paySign":"","signType":"RSA","nonceStr":""}
     */
    @Override
    public Map<String, String> createJsapi(Long userId, String paymentType, String orderNo) {
        //1.保存封装本地交易记录
        PaymentInfo paymentInfo = paymentInfoService.savePaymentInfo(paymentType, orderNo);
        //支付状态：1401-未支付 1402-已支付
        String paymentStatus = paymentInfo.getPaymentStatus();
        //判断本地交易支付状态
        if (SystemConstant.PAYMENT_STATUS_PAID.equals(paymentStatus)) {
            throw new GuiguException(500, "当前交易已支付");
        }

        //2.TODO 对接微信支付接口获取小程序拉起微信支付相关参数
        //2.1 构建支付service对象
        JsapiServiceExtension service =
                new JsapiServiceExtension.Builder().config(config).build();

        //2.2 创建预支付请求对象 用于产生微信交易
        PrepayRequest request = new PrepayRequest();
        Amount amount = new Amount();
        //TODO 开发中将应付金额设置为1分
        amount.setTotal(1);
        //amount.setTotal(paymentInfo.getAmount());
        request.setAmount(amount);
        request.setAppid(wxPayV3Config.getAppid());
        request.setMchid(wxPayV3Config.getMerchantId());
        request.setDescription(paymentInfo.getContent());
        request.setNotifyUrl(wxPayV3Config.getNotifyUrl());
        request.setOutTradeNo(paymentInfo.getOrderNo());
        //TODO 小程序还未上线，只允许应用下开发者用户付款 设置付款者唯一标识
        Payer payer = new Payer();
        payer.setOpenid("odo3j4qp-wC3HVq9Z_D9C0cOr0Zs");
        request.setPayer(payer);
        //2.3 调用下单方法，得到响应，封装调起支付的参数
        PrepayWithRequestPaymentResponse response = service.prepayWithRequestPayment(request);
        if (response != null) {
            String timeStamp = response.getTimeStamp();
            String nonceStr = response.getNonceStr();
            String packageVal = response.getPackageVal();
            String signType = response.getSignType();
            String paySign = response.getPaySign();
            Map<String, String> map = new HashMap<>();
            map.put("timeStamp", timeStamp);
            map.put("nonceStr", nonceStr);
            map.put("package", packageVal);
            map.put("signType", signType);
            map.put("paySign", paySign);
            return map;
        }
        return null;
    }

    /**
     * 查询订单交易状态
     *
     * @param orderNo
     * @return
     */
    @Override
    public boolean queryPayStatus(String orderNo) {
        //1.构建支付service对象
        JsapiServiceExtension service =
                new JsapiServiceExtension.Builder().config(config).build();
        //2.根据商户侧订单编号查询交易
        QueryOrderByOutTradeNoRequest request = new QueryOrderByOutTradeNoRequest();
        request.setMchid(wxPayV3Config.getMerchantId());
        request.setOutTradeNo(orderNo);
        Transaction transaction = service.queryOrderByOutTradeNo(request);
        //3.判断交易状态
        if (transaction != null) {
            log.info("订单对应交易结果:{}" + transaction);
            Transaction.TradeStateEnum tradeState = transaction.getTradeState();
            if (Transaction.TradeStateEnum.SUCCESS == tradeState) {
                return true;
            }
        }
        return false;
    }
}
